package cd.casic.ci.process.process.service.sast.impl;

import cd.casic.ci.process.constant.SastUrlConstant;
import cd.casic.ci.process.dto.resp.sast.TokenResp;
import cd.casic.ci.process.process.service.sast.SastService;
import cd.casic.ci.process.properties.SastProperties;
import cd.casic.framework.redis.core.RedisTemplateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

import static cd.casic.ci.process.constant.SastUrlConstant.*;

@Service
public class SastServiceImpl implements SastService {
    @Resource
    private RedisTemplateUtils redisTemplateUtils;
    @Resource
    private SastProperties sastProperties;
    @Resource
    private RestTemplate restTemplate;
    public static final String tokenPrefix = "Bearer";
    public static final String tokenHeaderKey = "authorization";
    public static final String REDIS_SAST_TOKEN_KEY = "REDIS_SAST_TOKEN_KEY";

    private TokenResp getTokenRemote(){
        HttpHeaders httpHeaders = new HttpHeaders();
        HttpEntity<SastProperties> httpEntity = new HttpEntity<SastProperties>(sastProperties,httpHeaders);
        ResponseEntity<String> exchange = restTemplate.exchange(baseUrl+getToken, HttpMethod.POST, httpEntity, String.class, new HashMap<>());
        String body = exchange.getBody();
        JSONObject bodyObject = JSON.parseObject(body);
        TokenResp tokenResp = new TokenResp();
        tokenResp.setAccessToken(bodyObject.getString("access_token"));
        tokenResp.setExpiresIn(bodyObject.getInteger("expires_in"));
        tokenResp.setRefreshExpiresIn(bodyObject.getInteger("refresh_expires_in"));
        tokenResp.setRefreshToken(bodyObject.getString("refresh_token"));
        tokenResp.setTokenType(bodyObject.getString("token_type"));
        return tokenResp;
    }

    @Override
    public String getToken() {
        Object o = redisTemplateUtils.get(REDIS_SAST_TOKEN_KEY);
        String token = o instanceof String ? ((String) o) : "";
        if (StringUtils.isEmpty(token)) {
            synchronized(this){
                o = redisTemplateUtils.get(REDIS_SAST_TOKEN_KEY);
                token = o instanceof String ? ((String) o) : "";
                if (StringUtils.isEmpty(token)) {
                    TokenResp tokenRemote = getTokenRemote();
                    String accessToken = tokenRemote.getAccessToken();
                    redisTemplateUtils.set(REDIS_SAST_TOKEN_KEY,accessToken,tokenRemote.getExpiresIn()*1000);
                    token = accessToken;
                }
            }
        }
        return token;
    }
}
